property mySprite, getPDLError, myOrientation, myActivePeriod, myResetFlag, myInitialAngle, myMouseDown

on getBehaviorDescription me
  return "TURN TOWARDS MOUSE" & RETURN & RETURN & "Orients a sprite to face (or turn away from) the mouse pointer, even if the sprite is moving." & RETURN & RETURN & "You can choose whether the sprite always turns to face the mouse, or only when the mouse button is up or down. " & "When the mouse is in the other state, the will either return to its initial position, or remain in the position it was in when the mouse state changed." & RETURN & RETURN & "The graphic member is considered to face right when it is at rest." & RETURN & RETURN & "PERMITTED MEMBER TYPES:" & RETURN & "bitmap, Flash, vector shape" & RETURN & RETURN & "PARAMETERS:" & RETURN & "* Whether to turn toward or away from the mouse" & RETURN & "* When to turn (always, while mouse is up, or while mouse is down)" & RETURN & "* Whether to return to initial position or remain in new position"
end

on getBehaviorTooltip me
  return "Use with bitmap, Flash and vector shape members." & RETURN & RETURN & "Orients a sprite to face towards or away from the mouse pointer, even if the sprite is moving." & RETURN & RETURN & "You can make the sprite face the mouse at all times, or only when the mouse is up or down. " & "When the mouse is in the other state, the sprite can either revert to its original postion, or remain in the position it was in when the mouse state changed. " & "The member is considered to face right at rest."
end

on beginSprite me
  Initialize(me)
end

on prepareFrame me
  if Activate(me) then
    Turn(me)
  else
    if myResetFlag then
      if mySprite.rotation <> myInitialAngle then
        mySprite.rotation = myInitialAngle
      end if
    end if
  end if
end

on Initialize me
  mySprite = sprite(me.spriteNum)
  memberType = mySprite.member.type
  case memberType of
    #flash, #vectorShape:
      mySprite.scaleMode = #autoSize
  end case
  myMouseDown = the stillDown
  myInitialAngle = mySprite.rotation
  myOrientation = myOrientation = "away from the mouse"
  case myActivePeriod of
    "always":
      myActivePeriod = #always
    "when the mouse is clicked":
      myActivePeriod = #mouseClick
    "when the mouse is released":
      myActivePeriod = #mouseRelease
    "while the mouse is down":
      myActivePeriod = #mouseDown
    "while the mouse is up":
      myActivePeriod = #mouseUp
  end case
  myResetFlag = myResetFlag = "return to the initial position"
end

on Activate me
  newStatus = myMouseDown <> the stillDown
  if newStatus then
    myMouseDown = the stillDown
  end if
  case myActivePeriod of
    #always:
      return 1
    #mouseClick:
      return newStatus and the stillDown
    #mouseRelease:
      return newStatus and not (the stillDown)
    #mouseDown:
      return the stillDown
    #mouseUp:
      return not (the stillDown)
  end case
end

on Turn me
  if the frameLabel <> "pause" then
    slope = the mouseLoc - mySprite.loc
    angle = GetAngle(me, slope)
    if myOrientation then
      angle = angle + 180
    end if
    if mySprite.rotation <> angle then
      mySprite.rotation = angle
    end if
  end if
end

on GetAngle me, slope
  deltaH = slope[1]
  deltaV = slope[2]
  if deltaH then
    slope = float(deltaV) / deltaH
    angle = atan(slope)
    if deltaH < 0 then
      angle = angle + PI
    end if
  else
    if deltaV > 0 then
      angle = PI / 2
    else
      if deltaV < 0 then
        angle = 3 * PI / 2
      else
        angle = 0
      end if
    end if
  end if
  angle = angle * 180 / PI
  return angle
end

on isOKToAttach me, aSpriteType, aSpriteNum
  case aSpriteType of
    #graphic:
      return getPos([#bitmap, #flash, #vectorShape], sprite(aSpriteNum).member.type) <> 0
    #script:
      return 0
  end case
end

on getPropertyDescriptionList me
  if not (the currentSpriteNum) then
    exit
  end if
  return [#myOrientation: [#comment: "Turn", #format: #string, #range: ["towards the mouse", "away from the mouse"], #default: "towards the mouse"], #myActivePeriod: [#comment: EMPTY, #format: #string, #range: ["while the mouse is down", "while the mouse is up", "when the mouse is clicked", "when the mouse is released", "always"], #default: "while the mouse is down"], #myResetFlag: [#comment: "Otherwise", #format: #string, #range: ["return to the initial position", "remain in the new position"], #default: "return to the initial position"]]
end
